/*
 * Engine class
 * by hackervalley@free.fr
 * http://hackervalley.free.fr
 * June 2004
 * this program is licensed under the GPL.
 */

#include "Engine.hh"
#include "Container.hh"
#include "Tokenizer.hh"

#include <iostream>

extern "C"
{
#include <GL/gl.h>
#include <GL/glu.h>
}

using namespace FE;
using namespace std;

Engine::Engine ()
{
	x0 = y0 = x1 = y1 = 0;
	StateDrawRectangle = false;
	StateMove = false;
}

Engine::~Engine ()
{
	VertexSet.clear ();
}

void
Engine::OpenFile (char *path)
{
	char buffer[256];
	string inputstring;

	ifstream readfile (path, ios::in);
	if (!readfile.is_open ())
	{
		cout << "Error opening file" << endl;
	}
	else
	{
		/* Clear all container */
		VertexSet.clear();
		
		/* Tokenize file */
		/* Line per Line */
		while (!readfile.eof ())
		{
			/* getline returns buff on success, and the NULL pointer on end-of-file */
			if (readfile.getline (buffer, 256) != NULL)
			{
				inputstring.clear ();
				inputstring = buffer;

				/* Tokenize line */
				vector < string > Result;
				Result.clear ();
				CTokenizer < CSeparator >::Tokenize (Result,
								     inputstring,
								     CSeparator
								     ("() ={},;"));

				/* Create Objects */
				/* Point(1) = {1,1,1,1}; */
				if (Result[0] == "Point")
				{
					Vertex <> *p1;
					p1 = new Vertex <>;
					p1->Identifier = atoi(Result[1].c_str());
					p1->x = atof(Result[2].c_str());
					p1->y = atof (Result[3].
							       c_str ());
					p1->z = atof (Result[4].
							       c_str ());
					p1->lc = atof(Result[5].c_str());

					cout << "Vertex create " << p1->Identifier << " " << p1->
						x << " " << p1->
						y << " " << p1->z << " " << p1-> lc << endl;
					VertexSet.push_back (p1);

					cout << " Size of VertexSet " <<
						VertexSet.size () << endl;
				}
			}
		}

		cout << "Read File:" << WorkFilename << endl;
	}
	readfile.close ();
}

void Engine::SaveFile(char *path)
{
	ofstream writefile (path, ios::out | ios::trunc);
	
	if (!writefile.is_open ())
	{
		cout << "Error opening file" << endl;
	}
	else
	{
		/* Store Point(1) = {1,1,1,1}; */
		for (VertexSetIterator = VertexSet.begin ();
	     VertexSetIterator != VertexSet.end (); VertexSetIterator++)
	{
		writefile << "Point(" << (*VertexSetIterator)->Identifier << ") = {"<< (*VertexSetIterator)->x << "," << (*VertexSetIterator)->y << "," << (*VertexSetIterator)->z << "," << (*VertexSetIterator)->lc << "};"<< endl; 
	}
}
	writefile.close();
}

void
Engine::draw ()
{
	glBegin (GL_POINTS);
	for (VertexSetIterator = VertexSet.begin ();
	     VertexSetIterator != VertexSet.end (); VertexSetIterator++)
	{
		if ((*VertexSetIterator)->IsSelect)
			glColor3f (1.0, 0.0, 0.0);

		else
			glColor3f (1.0, 1.0, 1.0);

		glVertex3d ((*VertexSetIterator)->x, (*VertexSetIterator)->y,
			    (*VertexSetIterator)->z);

	}
	glEnd ();

}

void
Engine::drawRectangle (int &x0, int &y0, int &x1, int &y1)
{
	GLint viewport[4];
	GLdouble mvmatrix[16], projmatrix[16];
	GLdouble wx0, wy0, wz0;	/* returned world x0, y0, z0 coords, first rectangle corner  */
	GLdouble wx1, wy1, wz1;	/* returned world x1, y1, z1 coords, second rectangle corner */
	GLdouble winx, winy, winz;	/* retruned window coords */

	glGetIntegerv (GL_VIEWPORT, viewport);
	glGetDoublev (GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev (GL_PROJECTION_MATRIX, projmatrix);

	/* Project Origin (0,0,0) to have winz */
	gluProject (0, 0, 0, mvmatrix, projmatrix, viewport, &winx, &winy,
		    &winz);

	/* z Mouse is winz */
	gluUnProject ((GLdouble) x0, (GLdouble) y0, winz,
		      mvmatrix, projmatrix, viewport, &wx0, &wy0, &wz0);

	/* z Mouse is winz */
	gluUnProject ((GLdouble) x1, (GLdouble) y1, winz,
		      mvmatrix, projmatrix, viewport, &wx1, &wy1, &wz1);

	/* Draw OpenGL rectangle */
	/* Remarks OpenGL Axis Y and Fltk Window are opposite */
	glColor3d (0.7, 0.7, 0.7);
	glBegin (GL_LINE_LOOP);
	glVertex3d (wx0, -wy0, wz0);
	glVertex3d (wx1, -wy0, wz0);
	glVertex3d (wx1, -wy1, wz0);
	glVertex3d (wx0, -wy1, wz0);
	glEnd ();

}

void
Engine::DrawMoveLine (int &x0, int &y0, int &x1, int &y1)
{
	GLint viewport[4];
	GLdouble mvmatrix[16], projmatrix[16];
	GLdouble wx0, wy0, wz0;	/* returned world x0, y0, z0 coords, first rectangle corner  */
	GLdouble wx1, wy1, wz1;	/* returned world x1, y1, z1 coords, second rectangle corner */
	GLdouble winx, winy, winz;	/* retruned window coords */

	glGetIntegerv (GL_VIEWPORT, viewport);
	glGetDoublev (GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev (GL_PROJECTION_MATRIX, projmatrix);

	/* Project Origin (0,0,0) to have winz */
	gluProject (0, 0, 0, mvmatrix, projmatrix, viewport, &winx, &winy,
		    &winz);

	/* z Mouse is winz */
	gluUnProject ((GLdouble) x0, (GLdouble) y0, winz,
		      mvmatrix, projmatrix, viewport, &wx0, &wy0, &wz0);

	/* z Mouse is winz */
	gluUnProject ((GLdouble) x1, (GLdouble) y1, winz,
		      mvmatrix, projmatrix, viewport, &wx1, &wy1, &wz1);

	/* Draw OpenGL rectangle */
	/* Remarks OpenGL Axis Y and Fltk Window are opposite */
	glColor3d (0.7, 0.7, 0.7);
	glBegin (GL_LINES);
	glVertex3d (wx0, -wy0, wz0);
	glVertex3d (wx1, -wy1, wz0);
	glEnd ();

}

void
Engine::createvertex (int &x, int &y)
{
	GLint viewport[4];
	GLdouble mvmatrix[16], projmatrix[16];
	GLdouble wx, wy, wz;	/* returned world x, y, z coords  */
	GLdouble winx, winy, winz;	/* returned window coords */

	glGetIntegerv (GL_VIEWPORT, viewport);
	glGetDoublev (GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev (GL_PROJECTION_MATRIX, projmatrix);

	/* Project Origin (0,0,0) to have winz */
	gluProject (0, 0, 0, mvmatrix, projmatrix, viewport, &winx, &winy,
		    &winz);

	/* z Mouse is winz */
	gluUnProject ((GLdouble) x, (GLdouble) y, winz,
		      mvmatrix, projmatrix, viewport, &wx, &wy, &wz);
	
	/* Define Identifier */
	int id=0;

	if(VertexSet.size()!=0)
	{
		id = (VertexSet.back()->Identifier) + 1;
	}
	
	/* Define Vertex */
	Vertex <> *p1;
	p1 = new Vertex <>;
	p1->Identifier = id;
	p1->x = wx;
	p1->y = -wy;
	p1->z = wz;
	p1->lc = 0.1;

	cout << "Vertex create " << p1->x << " " << p1->y << " " << p1->
		z << endl;

	VertexSet.push_back (p1);

	cout << " Size of VertexSet " << VertexSet.size () << endl;

}

void
Engine::selectvertex (int &x, int &y)
{
	GLdouble zone = 5;
	GLint viewport[4];
	GLdouble mvmatrix[16], projmatrix[16];
	GLdouble winx, winy, winz;	/*  returned window x, y, z coords  */

	glGetIntegerv (GL_VIEWPORT, viewport);
	glGetDoublev (GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev (GL_PROJECTION_MATRIX, projmatrix);

	for (VertexSetIterator = VertexSet.begin ();
	     VertexSetIterator != VertexSet.end (); VertexSetIterator++)
	{
		/* Remarks OpenGL Axis Y and Fltk Window are opposite */
		gluProject ((GLdouble) (*VertexSetIterator)->x,
			    -((GLdouble) (*VertexSetIterator)->y),
			    (GLdouble) (*VertexSetIterator)->z, mvmatrix,
			    projmatrix, viewport, &winx, &winy, &winz);

		if (winx > (x - zone) && winx < (x + zone)
		    && winy > (y - zone) && winy < (y + zone))
		{
			// Change State of vertex
			(*VertexSetIterator)->IsSelect =
				!(*VertexSetIterator)->IsSelect;

			cout << "Vertex state " << (*VertexSetIterator)->
				IsSelect << endl;
		}
	}
}

void
Engine::RectangleSelectVertex (int &x0, int &y0, int &x1, int &y1)
{
	/* Unused GLdouble zone = 5; */
	GLint viewport[4];
	GLdouble mvmatrix[16], projmatrix[16];
	GLdouble winx, winy, winz;	/*  returned window x, y, z coords  */

	glGetIntegerv (GL_VIEWPORT, viewport);
	glGetDoublev (GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev (GL_PROJECTION_MATRIX, projmatrix);

	for (VertexSetIterator = VertexSet.begin ();
	     VertexSetIterator != VertexSet.end (); VertexSetIterator++)
	{
		/* Remarks OpenGL Axis Y and Fltk Window are opposite */
		gluProject ((GLdouble) (*VertexSetIterator)->x,
			    -((GLdouble) (*VertexSetIterator)->y),
			    (GLdouble) (*VertexSetIterator)->z, mvmatrix,
			    projmatrix, viewport, &winx, &winy, &winz);

		if ((winx > x0 && winx < x1 && winy > y0 && winy < y1)
		    || (winx > x1 && winx < x0 && winy > y1 && winy < y0)
		    || (winx > x0 && winx < x1 && winy > y1 && winy < y0)
		    || (winx > x1 && winx < x0 && winy > y0 && winy < y1))
		{
			// Change State of vertex
			(*VertexSetIterator)->IsSelect =
				!(*VertexSetIterator)->IsSelect;

			cout << "Vertex state " << (*VertexSetIterator)->
				IsSelect << endl;
		}

	}
}


void
Engine::DeleteSelectVertex ()
{
	/* Create a container of iterator from vertex selected */

	list < list < Vertex <> *>::iterator > tmpVertexSetIterator;
	list < list <
		Vertex <> *>::iterator >::
		iterator tmpVertexSetIteratorIterator;

	for (VertexSetIterator = VertexSet.begin ();
	     VertexSetIterator != VertexSet.end (); VertexSetIterator++)
	{
		if ((*VertexSetIterator)->IsSelect == true)
		{
			tmpVertexSetIterator.push_back (VertexSetIterator);
			cout << " Address " << *VertexSetIterator << endl;
		}
	}

	/* Erase vertex selected */

	for (tmpVertexSetIteratorIterator = tmpVertexSetIterator.begin ();
	     tmpVertexSetIteratorIterator != tmpVertexSetIterator.end ();
	     tmpVertexSetIteratorIterator++)
	{
		VertexSet.erase (*tmpVertexSetIteratorIterator);

		cout << " Size of VertexSet " << VertexSet.size () << endl;
	}

}

void
Engine::MoveSelected (int &x0, int &y0, int &x1, int &y1)
{
	/* Define vector of translation */
	GLint viewport[4];
	GLdouble mvmatrix[16], projmatrix[16];
	GLdouble wx0, wy0, wz0;	/* returned world x0, y0, z0 coords, first rectangle corner  */
	GLdouble wx1, wy1, wz1;	/* returned world x1, y1, z1 coords, second rectangle corner */
	GLdouble winx, winy, winz;	/* retruned window coords */

	glGetIntegerv (GL_VIEWPORT, viewport);
	glGetDoublev (GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev (GL_PROJECTION_MATRIX, projmatrix);

	/* Project Origin (0,0,0) to have winz */
	gluProject (0, 0, 0, mvmatrix, projmatrix, viewport, &winx, &winy,
		    &winz);

	/* z Mouse is winz */
	gluUnProject ((GLdouble) x0, (GLdouble) y0, winz,
		      mvmatrix, projmatrix, viewport, &wx0, &wy0, &wz0);

	/* z Mouse is winz */
	gluUnProject ((GLdouble) x1, (GLdouble) y1, winz,
		      mvmatrix, projmatrix, viewport, &wx1, &wy1, &wz1);

	/* Translate selected */
	for (VertexSetIterator = VertexSet.begin ();
	     VertexSetIterator != VertexSet.end (); VertexSetIterator++)
	{
		if ((*VertexSetIterator)->IsSelect == true)
		{
			(*VertexSetIterator)->x += wx1 - wx0;
			(*VertexSetIterator)->y -= wy1 - wy0;
		}
	}

}
