/*
 * Gui class
 * by hackervalley@free.fr
 * http://hackervalley.free.fr
 * May 2004
 * this program is licensed under the GPL.
 */

#include "Gui.hh"

using namespace FE;
using namespace std;

// Constructor

Gui::Gui ()
{
// initialize static windows
	mainWindow = NULL;
	mainMenuBar = NULL;
	glWindow = NULL;
	statusBar = NULL;
	geoBar = NULL;
	aboutWindow = NULL;

	width = 800;
	height = 600;

	// Set Fltk theme
	Fl::scheme ("plastic");

}

Gui::~Gui ()
{
	delete mainWindow;
	delete mainMenuBar;
	delete glWindow;
	delete statusBar;
	delete geoBar;
	delete aboutWindow;
}

// Callbacks
// File Menu
void
Gui::open_cb(Fl_Widget *w, void *data)
{
	Gui *o = (Gui *) data;
	o->open_cb2(w, data);
}

void
Gui::open_cb2(Fl_Widget *, void *)
{
	char *openfile = NULL;
	openfile = fl_file_chooser("Open File","*.geo", "");
 	 if (openfile != NULL) { glWindow->engine->WorkFilename = openfile;}
	 
	 glWindow->engine->OpenFile(openfile);
}

void
Gui::saveas_cb(Fl_Widget *w, void *data)
{
	Gui *o = (Gui *) data;
	o->saveas_cb2(w, data);
}

void
Gui::saveas_cb2(Fl_Widget *, void *)
{
	char *savefile;
	savefile = NULL;
	savefile = fl_file_chooser("Save File As","*.geo", "");
 	 if (savefile != NULL) { glWindow->engine->WorkFilename = savefile;}
	 
	 glWindow->engine->SaveFile(savefile);
}

void
Gui::file_quit_cb (Fl_Widget *, void *)
{
	exit (0);
}

// Help Menu
void
Gui::help_about_cb (Fl_Widget * w, void *data)
{
	Gui *o = (Gui *) data;
	o->help_about_cb2 (w, data);
}

void
Gui::help_about_cb2 (Fl_Widget * w, void *)
{
	create_about_window ();
}

// geoBar callback
void
Gui::create_Vertex_cb (Fl_Widget * w, void *data)
{
	Gui *o = (Gui *) data;
	o->create_Vertex_cb2 ();
}

void
Gui::create_Vertex_cb2 ()
{
	glWindow->createvertex = true;
}


void
Gui::create_main_window ()
{
	if (mainWindow)
	{
		mainWindow->show ();
		return;
	}

	mainWindow = new Fl_Window (width, height, "FE");

	// Define resizable limits
	mainWindow->size_range (width, height, 0, 0, 0, 0, 0);

	mainWindow->begin ();

	// mainMenuBar
	mainMenuBar = new Fl_Menu_Bar (0, 0, mainWindow->w (), 30);

	Fl_Menu_Item menutable[] = {
		{"&File", FL_ALT + 'f', 0, 0, FL_SUBMENU}
		,
		{"&Open", FL_ALT + 'O', (Fl_Callback *)open_cb, (void *) this}
		,
		{"&Save As", FL_ALT + 's', (Fl_Callback *)saveas_cb, (void *) this,  FL_MENU_DIVIDER}
		,
		{"&Quit", FL_ALT + 'q', (Fl_Callback *) file_quit_cb, 0,
		 FL_MENU_DIVIDER}
		,
		{0}
		,
		{"&Help", FL_ALT + 'h', 0, 0, FL_SUBMENU}
		,
		{"&About", FL_ALT + 'a', (Fl_Callback *) help_about_cb,
		 (void *) this},
		{0},
		{0}
	};

	mainMenuBar->copy (menutable);

	// statusBar
	statusBar =
		new Fl_Group (mainWindow->x (), mainWindow->h () - 30,
			      mainWindow->w (), 30, "StatusBar");

	statusBar->begin ();
	Fl_Box *statusBox =
		new Fl_Box (statusBar->x (), statusBar->y (), statusBar->w (),
			    statusBar->h (), NULL);
	statusBox->box (FL_DOWN_BOX);
	statusBox->align (FL_ALIGN_LEFT | FL_ALIGN_INSIDE | FL_ALIGN_CLIP);
	statusBox->label ("Status Box");
	statusBar->end ();

	// geoBar
	geoBar = new Fl_Group (mainWindow->w () - 30, mainWindow->y () + 30,
			       30, mainWindow->h () - 60, NULL);

	geoBar->begin ();

	Fl_Box *geoBox =
		new Fl_Box (geoBar->x (), geoBar->y (), 30, geoBar->h (),
			    NULL);
	geoBox->box (FL_DOWN_BOX);
	geoBox->align (FL_ALIGN_LEFT | FL_ALIGN_INSIDE | FL_ALIGN_CLIP);

	Fl_Button *createVertex =
		new Fl_Button (geoBox->x (), geoBox->y (), 30, 30, "Pt");
	createVertex->callback ((Fl_Callback *) create_Vertex_cb,
				(void *) this);

	// Create additional box for the group who consume resize
	Fl_Box *additionalBox =
		new Fl_Box (geoBar->x (), geoBox->y () + 220, geoBar->w (),
			    60, NULL);
	geoBar->resizable (additionalBox);

	geoBar->end ();

	// glWindow
	glWindow = new OpenGLWindow (mainWindow->x (), mainWindow->y () + 30,
				     mainWindow->w () - 30,
				     mainWindow->h () - 60, "OpenGL Window");

	// Make resizable group
	mainWindow->add (glWindow);
	mainWindow->resizable (glWindow);

	mainWindow->end ();

	mainWindow->show ();

}

void
Gui::OK_cb (Fl_Widget * w, void *data)
{
	Fl_Window *o = (Fl_Window *) data;
	o->hide ();
}

void
Gui::create_about_window ()
{
	if (aboutWindow)
	{
		aboutWindow->show ();
		return;
	}

	aboutWindow = new Fl_Window (200, 200, "About FE");

	aboutWindow->begin ();

	Fl_Multiline_Output *AboutOutput;
	AboutOutput = new Fl_Multiline_Output (0, 0, 200, 170, "AboutOutput");

	AboutOutput->
		value
		("\n Finite Element\n\n under GPL\n\n http://hackervalley.free.fr");
	AboutOutput->align (FL_ALIGN_CENTER);

	Fl_Return_Button *ReturnButton;
	ReturnButton = new Fl_Return_Button (70, 170, 60, 30, "OK");

	ReturnButton->callback ((Fl_Callback *) OK_cb,
				ReturnButton->parent ());

	aboutWindow->end ();

	aboutWindow->show ();
}



int
Gui::run ()
{
	return Fl::run ();
}
